<?php

namespace FiloBlu\Storelocator\Controller\Adminhtml\Stores;

use FiloBlu\Storelocator\Block\Adminhtml\Stores\Edit\Tab\Image;
use Magento\Backend\App\Action;
use Magento\Backend\Model\View\Result\Page;
use Magento\Backend\Model\View\Result\Redirect;
use Magento\Framework\Registry;
use Magento\Framework\View\Result\LayoutFactory;
use Magento\Framework\View\Result\PageFactory;

class Images extends Action
{
	/**
	 * Core registry
	 *
	 * @var Registry
	 */
	protected $_coreRegistry = null;

	/**
	 * @var PageFactory
	 */
	protected $resultLayoutFactory;

	/**
	 * @param Action\Context $context
	 * @param PageFactory $resultPageFactory
	 * @param Registry $registry
	 */
	public function __construct(
		Action\Context              $context,
		LayoutFactory               $resultLayoutFactory,
		Registry $registry
	)
	{
		$this->resultLayoutFactory = $resultLayoutFactory;
		$this->_coreRegistry = $registry;
		parent::__construct($context);
	}

	/**
	 * Edit Storelocator store
	 *
	 * @return Page|Redirect
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 */
	public function execute()
	{
		$id = $this->getRequest()->getParam('store_id');

		$model = $this->_objectManager->create('FiloBlu\Storelocator\Model\Stores');

		if ($id) {
			$model->load($id);
			if (!$model->getId()) {
				$this->messageManager->addError(__('This store no longer exists.'));
				/** \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
				$resultRedirect = $this->resultRedirectFactory->create();

				return $resultRedirect->setPath('*/*/');
			}
		}

		$data = $this->_objectManager->get('Magento\Backend\Model\Session')->getFormData(true);
		if (!empty($data)) {
			$model->setData($data);
		}

		$this->_coreRegistry->register('storelocator_store', $model);

		$resultLayout = $this->resultLayoutFactory->create();
		/** @var Image $imagesBlock */
		$imagesBlock = $resultLayout->getLayout()->getBlock('store.edit.tab.image');
		if ($imagesBlock) {
			$imagesBlock->setStoreImages($this->getRequest()->getPost('store_images', null));
		}
		return $resultLayout;
	}

	/**
	 * {@inheritdoc}
	 */
	protected function _isAllowed()
	{
		return $this->_authorization->isAllowed('FiloBlu_Storelocator::storelocator_save');
	}
}
