<?php

namespace FiloBlu\Storelocator\Controller\Index;

use Exception;
use FiloBlu\Storelocator\Model\StoresFactory;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Area;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filesystem;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Framework\View\Result\Page;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;

class Appointment extends Action
{
	/**
	 * @return Redirect|Page
	 */

	protected $context;
	protected $_scopeConfig;
	protected $storesFactory;
	/**
	 * @var TransportBuilder
	 */
	protected $_transportBuilder;
	private $fileSystem;


	/**
	 * @var StoreManagerInterface
	 */

	/**
	 * @param Context $context
	 * @param Filesystem $fileSystem
	 * @param TransportBuilder $transportBuilder
	 * @param StoreManagerInterface $_storeManager
	 * @param ScopeConfigInterface $scopeConfig
	 * @param StoresFactory $storesFactory
	 * @param JsonFactory $resultJsonFactory
	 */
	public function __construct(
		Context               $context,
		Filesystem            $fileSystem,
		TransportBuilder      $transportBuilder,
		StoreManagerInterface $_storeManager,
		ScopeConfigInterface  $scopeConfig,
		StoresFactory         $storesFactory,
		JsonFactory           $resultJsonFactory
	)
	{
		parent::__construct($context);
		$this->fileSystem = $fileSystem;
		$this->_transportBuilder = $transportBuilder;
		$this->_storeManager = $_storeManager;
		$this->_scopeConfig = $scopeConfig;
		$this->storesFactory = $storesFactory;
		$this->_resultJsonFactory = $resultJsonFactory;
	}

	public function execute()
	{

		$resultJson = $this->_resultJsonFactory->create();
		$post = $this->getRequest()->getPostValue();


		$shopId = $this->getRequest()->getParam('id');
		$shop = $this->storesFactory->create()->load($shopId);
		$shopData = $shop->getdata();
		$shopDataArray = $shopData;


		$storeName = $this->_scopeConfig->getValue(
			'filoblu_storelocator_section/general/appointment__contact_name',
			ScopeInterface::SCOPE_STORE
		);

		$storeEmail = $this->_scopeConfig->getValue(
			'filoblu_storelocator_section/general/appointment__contact_email',
			ScopeInterface::SCOPE_STORE
		);

		$appointmentEmailTemplate = $this->_scopeConfig->getValue(
			'filoblu_storelocator_section/general/appointment_email_template',
			ScopeInterface::SCOPE_STORE
		);

		$appointmentEmailTemplateToCustomer = $this->_scopeConfig->getValue(
			'filoblu_storelocator_section/general/appointment_email_to_customer_template',
			ScopeInterface::SCOPE_STORE
		);

		$emailToShop = $this->_scopeConfig->getValue(
			'filoblu_storelocator_section/general/appointment_email_to_shop',
			ScopeInterface::SCOPE_STORE
		);

		$emailCcToContact = $this->_scopeConfig->getValue(
			'filoblu_storelocator_section/general/appointment_email_cc_contact',
			ScopeInterface::SCOPE_STORE
		);

		// Email sender to store

		$fromEmailToStore = $storeEmail;
		$fromNameToStore = $storeName;

		$templateVarsToStore = [
			'store'               => $this->_storeManager->getStore(),
			'customer_name'       => $post['appointment_name'],
			'customer_lastname'   => $post['appointment_lastname'],
			'customer_phone'      => $post['appointment_phone'],
			'customer_email'      => $post['appointment_email'],
			'chosen_date'         => $post['appointment_date'],
			'time_slot'           => $post['appointment_time_slot'],
			'appointment_message' => $post['appointment_message'],
			'appointment_hours'   => $post['appointment_hours'],
			'product_name'        => $post['product_name'],
			'product_sku'         => $post['product_sku'],
			'product_price'       => $post['product_price'],
			'product_image'       => $post['product_image'],
			'wishlist_products'   => $post['wishlist_products']
		];

		$fromToStore = ['email' => $fromEmailToStore, 'name' => $fromNameToStore];
		$toToStore = ($emailToShop && $shopData['email']) ? $shopData['email'] : $storeEmail;
		$emailCcToStore = $emailCcToContact ? $storeEmail : false;


		$templateOptions = [
			'area'  => Area::AREA_FRONTEND,
			'store' => Store::DEFAULT_STORE_ID
		];

		try {
			if ($emailCcToStore) {
				$transport = $this->_transportBuilder->setTemplateIdentifier($appointmentEmailTemplate)
					->setTemplateOptions($templateOptions)
					->setTemplateVars($templateVarsToStore)
					->setFrom($fromToStore)
					->addTo($toToStore)
					->addCc($emailCcToStore)
					->getTransport();
			} else {
				$transport = $this->_transportBuilder->setTemplateIdentifier($appointmentEmailTemplate)
					->setTemplateOptions($templateOptions)
					->setTemplateVars($templateVarsToStore)
					->setFrom($fromToStore)
					->addTo($toToStore)
					->getTransport();
			}
			$transport->sendMessage();

		} catch (LocalizedException $e) {
			return $resultJson->setData(['success' => false, 'error' => true, 'message' => __('There was a problem with the request: %1', $e->getMessage())]);
		} catch (Exception $e) {
			return $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
		}

		//Email sender to customer

		$templateVarsToCustomer = [
			'store'             => $this->_storeManager->getStore(),
			'customer_name'     => $post['appointment_name'],
			'store_name'        => $shopDataArray['title'],
			'store_address'     => $shopDataArray['street'],
			'store_postcode'    => $shopDataArray['postcode'],
			'store_city'        => $shopDataArray['city'],
			'store_phone'       => $shopDataArray['phone_1'],
			'store_email'       => $shopDataArray['email'],
			'product_name'      => $post['product_name'],
			'product_sku'       => $post['product_sku'],
			'product_price'     => $post['product_price'],
			'product_image'     => $post['product_image'],
			'chosen_date'       => $post['appointment_date'],
			'time_slot'         => $post['appointment_time_slot'],
			'wishlist_products' => $post['wishlist_products']
		];

		$fromEmailToCustomer = $storeEmail;
		$fromNameToCustomer = $storeName;

		$fromToCustomer = ['email' => $fromEmailToCustomer, 'name' => $fromNameToCustomer];

		$toToCustomer = $post['appointment_email'];

		try {
			$transport = $this->_transportBuilder->setTemplateIdentifier($appointmentEmailTemplateToCustomer)
				->setTemplateOptions($templateOptions)
				->setTemplateVars($templateVarsToCustomer)
				->setFrom($fromToCustomer)
				->addTo($toToCustomer)
				->getTransport();
			$transport->sendMessage();

		} catch (Exception $e) {
			return $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
		}

		$resultJson->setData(['success' => true, 'error' => false, 'message' => __('Apointment request recived, a recap email has been sent to you')]);

		return $resultJson;
	}
}