<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Storelocator\Controller\PickUpInStore;

use FiloBlu\Storelocator\Api\PickUpInStoreManagementInterface;
use Magento\Framework\App\ObjectManager;
use \Magento\Framework\Exception\NotFoundException;
use Magento\Rma\Model\Rma;
use Monolog\Logger;

/**
 * Responsible for loading page content.
 *
 * This is a basic controller that only loads the corresponding layout file. It may duplicate other such
 * controllers, and thus it is considered tech debt. This code duplication will be resolved in future releases.
 */
class ConfirmReceivedAction extends \Magento\Framework\App\Action\Action
{
    protected $jsonResultFactory;
    protected $helper;
    protected $_order;
    protected $_transportBuilder;
    protected $_storeManager;
    protected $_inlineTranslation;
    protected $_paymentHelper;
    protected $_addressRenderer;
    protected $_coreRegistry;
    protected $_resourceConnection;
    protected $_rma;
    protected $_rmaHistory;
    /**
     * @var \Magento\Rma\Helper\Data
     */
    protected $_rmaData;

    protected $_status = PickUpInStoreManagementInterface::STATUS_DELIVERED_IN_STORE;

    public function __construct(
        \Magento\Framework\App\Action\Context $context ,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory ,
        \FiloBlu\Storelocator\Helper\Data $helper ,
        \Magento\Sales\Api\Data\OrderInterface $order ,
        \Magento\Framework\Stdlib\DateTime\DateTime $date ,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder ,
        \Magento\Store\Model\StoreManagerInterface $storeManager ,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation ,
        \Magento\Payment\Helper\Data $paymentHelper ,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer ,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Rma\Model\Rma\Status\History $rmaHistory,
        \Magento\Rma\Helper\Data $rmaData
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;
        $this->_order = $order;
        $this->date = $date;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_paymentHelper = $paymentHelper;
        $this->_addressRenderer = $addressRenderer;
        $this->_coreRegistry = $registry;
        $this->_resourceConnection = $resourceConnection;
        $this->_rmaHistory = $rmaHistory;
        $this->_rmaData = $rmaData;
        parent::__construct($context);
    }


    public function execute()
    {

        $params = $this->getRequest()->getParams();
        $hash = isset($params['hash']) ? $params['hash'] : null;

        if ((!$this->helper->getPickupInStoreEnabled() && !$this->helper->getReturnToStoreEnabled())  || !isset($params["increment_id"]) || !$params["increment_id"]) {
            return $this->_forward('noRoute');
        }

        $this->_order->loadByIncrementId($params["increment_id"]);

        if ($this->helper->getReturnToStoreEnabled() && isset($params['rma_id'])) {

            /** @var Rma $rma */
            $model = ObjectManager::getInstance()->get(Rma::class);
            $collection = $model->getCollection();
            
            $this->_rma = $collection
                ->addFieldToFilter('order_increment_id', $params['increment_id'])
                ->addFieldToFilter('status', 'authorized')
                ->getLastItem();

            if ($this->_order->getId() && $this->_rma->getReturnToStoreHash() == $hash) {
                $resultJson = $this->resultJsonFactory->create();
                $resultJson->setData(["message" => __("You confirmed that the order ") . $params["increment_id"] . __(" was received.")]);
                $this->updateRma();
                $this->updateOrder();
                $this->_rma
                    ->setStatus(\FiloBlu\Storelocator\Model\Plugin\Rma\Source\Status::STATE_RECEIVED_IN_STORE)
                    ->setReturnToStoreReceivedDate($this->date->gmtDate())
                    ->setIsUpdate(1)
                    ->save();
                // Send mail to customer to inform that the package is available in store.
                $this->_eventManager->dispatch('filoblu_storelocator_pickupinstore_confirm_received' , ['order' => $this->_order]);

                return $resultJson;
            }
        } else {

            if ($this->_order->getId() && $this->_order->getPickupInStoreHash() == $hash && $this->_order->getShippingMethod() == "pickupinstore_pickupinstore") {
                $this->_order->setPickupInStoreConfirmReceivedDate($this->date->gmtDate());
                $this->_order->save();
                $resultJson = $this->resultJsonFactory->create();
                $resultJson->setData(["message" => __("You confirmed that the order ") . $params["increment_id"] . __(" was received.")]);
                $this->updateOrder();
                // Send mail to customer to inform that the package is available in store.
                $this->sendMail($this->_order);
                $this->_eventManager->dispatch('filoblu_storelocator_pickupinstore_confirm_received' , ['order' => $this->_order]);

                return $resultJson;
            }
        }

        return $this->_forward('noRoute');
    }

    /**
     *  Update Order-status
     */
    public function updateOrder()
    {

        $order = $this->_order;
        $order->setState(\Magento\Sales\Model\Order::STATE_COMPLETE , true)->save();
        $order->setStatus("delivered_in_store")->save();
        $order->addStatusToHistory($order->getStatus() , 'Pick Up In Store - The order arrived in the store');
        $order->save();
    }

    protected function sendMail($order)
    {
        if (!$this->helper->getConfig("carriers/pickupinstore/email_to_customer_enabled")) {
            return;
        }

        $this->_inlineTranslation->suspend();

        $billingAddress = $order->getBillingAddress();

        $templateVars = [
            'order' => $order ,
            'billing' => $billingAddress ,
            'payment_html' => $this->getPaymentHtml($order) ,
            'store' => $order->getStore() ,
            'formattedShippingAddress' => $this->getFormattedShippingAddress($order) ,
            'formattedBillingAddress' => $this->getFormattedBillingAddress($order) ,
        ];


        if ($this->helper->getReturnToStoreEnabled()) {

            if ($this->_rma) {
                $storeId = $this->_rma->getReturnToStoreId();

                if ($storeId) {
                    $data = $this->helper->getStoreDetails($storeId);
                    $returnAddress = $this->_rmaData->getReturnAddress('html', $data, $storeId);

                    $templateVars['formattedShippingAddress'] = $returnAddress;
                }
            }

        }

        $templateOptions = [
            'area' => \Magento\Framework\App\Area::AREA_FRONTEND ,
            'store' => $this->_storeManager->getStore()->getStoreId() ,
        ];

        try {
            $transport = $this->_transportBuilder
                ->setTemplateIdentifier($this->helper->getConfig("carriers/pickupinstore/email_to_customer_template"))
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVars)
                ->setFrom($this->helper->getConfig("carriers/pickupinstore/sender_email_to_customer_identity"))
                ->addTo($order->getCustomerEmail())
                ->getTransport();
            $transport->sendMessage();

            $this->_inlineTranslation->resume();

        } catch (\Exception $e) {
            $this->_inlineTranslation->resume();
        }
    }

    protected function getPaymentHtml($order)
    {
        return $this->_paymentHelper->getInfoBlockHtml(
            $order->getPayment() ,
            $this->_storeManager->getStore()->getId()
        );
    }

    protected function getFormattedShippingAddress($order)
    {
        return $order->getIsVirtual()
            ? null
            : $this->_addressRenderer->format($order->getShippingAddress() , 'html');
    }

    protected function getFormattedBillingAddress($order)
    {
        return $this->_addressRenderer->format($order->getBillingAddress() , 'html');
    }

    public function updateRma()
    {

        $connection = $this->_resourceConnection->getConnection();
        $tableName = $this->_resourceConnection->getTableName('magento_rma_item_entity');

        $query_rma_items = "SELECT * FROM " . $tableName . " WHERE rma_entity_id = " . $this->_rma->getData('entity_id');
        $result = $connection->fetchAll($query_rma_items);
        foreach ($result as $idx => $item) {
            $item_id = $item['entity_id'];
            $qty_request = $item['qty_requested'];

            if ((int)$qty_request > 0) {

                $approvedReturned = '';

                if ($this->helper->getUseStandardRmaMailFlow()) {
                    $approvedReturned = ", qty_approved = " . $qty_request . ", qty_returned = " . $qty_request;
                    //$approvedReturned = ", qty_approved = " . $qty_request . ", qty_returned = " . $qty_request;
                } else {
                    //if ($this->_rma->getOrigData('status') === 'authorized' && $this->_rma->getStatus() === 'received') {
                        $approvedReturned = ", qty_approved = " . $qty_request . ", qty_returned = " . $qty_request;
                        //$approvedReturned = ", qty_returned = " . $qty_request;
                    //}
                }

                $sql = "UPDATE " . $tableName . " SET status = '" . \FiloBlu\Storelocator\Model\Plugin\Rma\Source\Status::STATE_RECEIVED_IN_STORE . "', 
                qty_authorized = " . $qty_request . $approvedReturned . " WHERE entity_id = " . $item_id;


                $connection->query($sql);
            }
        }

        $comment = 'Return In Store - The package was picked up by store';
        $this->_rmaHistory->setRmaEntityId($this->_rma->getId());
        $this->_rmaHistory->setComment($comment);
        $this->_rmaHistory->saveComment($comment , false , true);

    }
}
