<?php

namespace FiloBlu\Storelocator\Model\Plugin\Rma;

use FiloBlu\Storelocator\Helper\Data;
use Magento\Framework\App\RequestInterface;
use Magento\Rma\Model\ResourceModel\Rma\CollectionFactory;

class Config
{
    /**
     * @var Data
     */
    protected $helper;
    /**
     * @var RequestInterface
     */
    protected $request;
    /**
     * @var CollectionFactory
     */
    protected $rmaFactory;

    public function __construct(
        Data $helper,
        RequestInterface $request,
        CollectionFactory $rmaFactory
    ) {
        $this->helper = $helper;
        $this->request = $request;
        $this->rmaFactory = $rmaFactory;
    }

    public function afterGetCopyTo(\Magento\Rma\Model\Config $subject, $result)
    {
        if (!$this->helper->getReturnToStoreEnabled()) {
            return $result;
        }

        $orderId = $this->request->getParam('order_id');
        $rmaId = $this->request->getParam('rma_id');

        if (!$orderId && !$rmaId) {
            return $result;
        }

        /** @var \Magento\Rma\Model\Rma $rma */
        $rma = $this->getRmaCollection($orderId);

        if ($rma->getStatus() !== 'pending') {
            return $result;
        }

        if ($this->helper->getReturnToStoreEnabled() && $rma->getReturnToStoreId()) {
            return null;
        }

        return $result;
    }

    public function getRmaCollection($orderId)
    {
        return $this->rmaFactory->create()->addFieldToSelect(
            '*'
        )->addFieldToFilter(
            'order_id',
            $orderId
        )->getLastItem();
    }
}